@@ -36,10 +36,13 @@ class Artifact(object):
         'name',
         'output_hash',
         'state',
-        'stdout'
+        'stdout',
+        'virtual'
     ]
 
     BINARY_EXTENSIONS = [
+         '.docx',
+        '.epub',
         '.gif',
         '.jpg',
         '.png',
@@ -49,6 +52,8 @@ class Artifact(object):
         '.gz',
         '.eot',
         '.ttf',
+        '.odt',
+        '.rtf',
         '.woff',
         '.sqlite',
         '.sqlite3',
@@ -143,11 +148,13 @@ class Artifact(object):
         """
         Set up an initial artifact (the first artifact in a document's filter chain).
         """
-        self._inputs = self.doc.input_artifacts()
+        self._inputs = {}
         self.binary_input = (self.doc.ext in self.BINARY_EXTENSIONS)
         self.binary_output = self.binary_input
         self.ext = self.doc.ext
         self.initial = True
+        self.virtual = self.doc.virtual
+        self.virtual_docs = self.doc.virtual_docs
 
         if self.args.has_key('final'):
             self.final = self.args['final']
@@ -200,20 +207,33 @@ class Artifact(object):
             self.final = self.filter_class.FINAL
 
     def setup_from_previous_artifact(self, previous_artifact):
-        for a in ['final', 'mtime', 'ctime', 'inode']:
+        for a in ['final', 'mtime', 'ctime', 'inode', 'virtual', 'virtual_docs']:
                 setattr(self, a, getattr(previous_artifact, a))
 
-        self._inputs.update(previous_artifact.inputs())
-        # Need to loop over each artifact's inputs in case extra ones have been
-        # added anywhere.
+        # Look for additional inputs in previous artifacts or previous
+        # artifacts' inputs.
         for k, a in previous_artifact.inputs().iteritems():
-            self._inputs.update(a.inputs())
+
+            if a.additional and not k in self._inputs:
+                self.log.debug("(%s) Adding additional artifact %s from %s" % (self.key, k, a.key))
+                self.add_input(k, a)
+            elif not k in self._inputs:
+                # We should have all other inputs already. Validate this.
+                raise Exception("Missing input %s" % k)
+
+            for kk, aa in a.inputs().iteritems():
+                if aa.additional and not kk in self._inputs:
+                    self.log.debug("(%s) Adding additional artifact %s from %s" % (self.key, kk, k))
+                    self.add_input(kk, aa)
+
         self.binary_input = previous_artifact.binary_output
         self.input_data_dict = previous_artifact.data_dict
         self.input_ext = previous_artifact.ext
         self.previous_artifact_filename = previous_artifact.filename()
         self.previous_artifact_filepath = previous_artifact.filepath()
         self.previous_canonical_filename = previous_artifact.canonical_filename(True)
+        self.previous_long_canonical_filename = previous_artifact.long_canonical_filename()
+        self.previous_websafe_key = previous_artifact.websafe_key()
 
         # The JSON output of previous artifact
         if not previous_artifact.binary_output:
@@ -225,10 +245,16 @@ class Artifact(object):
         else:
             next_inputs = None
 
-        self.ext = self.filter_class.output_file_extension(
-                previous_artifact.ext,
-                self.name,
-                next_inputs)
+        if self.args.has_key('ext'):
+            ext = self.args['ext']
+            if not ext.startswith("."):
+                ext = ".%s" % ext
+            self.ext = ext
+        else:
+            self.ext = self.filter_class.output_file_extension(
+                    previous_artifact.ext,
+                    self.name,
+                    next_inputs)
 
         self.binary_output = self.filter_class.BINARY
         if self.binary_output is None:
@@ -268,6 +294,18 @@ class Artifact(object):
             artifact.next_filter_name = next_filter_class.__name__
             artifact.next_filter_class = next_filter_class
 
+        # Set inputs from original document inputs.
+        artifact._inputs.update(artifact.doc.input_artifacts())
+        artifact.log.debug("add inputs from document %s: %s" % (artifact.doc.key(), ", ".join(artifact.doc.input_artifacts().keys())))
+
+        for k, a in artifact.doc.input_artifacts().iteritems():
+            if a.additional and not k in artifact._inputs:
+                artifact.add_input(k, a)
+
+            for kk, aa in a.inputs().iteritems():
+                if aa.additional and not kk in artifact._inputs:
+                    artifact.add_input(kk, aa)
+
         if previous_artifact:
             artifact.setup_from_previous_artifact(previous_artifact)
             artifact.setup_from_filter_class()
@@ -275,9 +313,11 @@ class Artifact(object):
             artifact.setup_initial()
 
         artifact.set_hashstring()
+
         return artifact
 
     def run(self):
+
         start = time.time()
 
         if self.controller_args['nocache'] or not self.is_complete():
@@ -293,10 +333,11 @@ class Artifact(object):
             try:
                 filter_instance.process()
             except Exception as e:
+                # TODO Clean this up, should all go to stderr probably.
                 print "Error occurred while running", self.key
                 x, y, tb = sys.exc_info()
                 print "Original traceback:"
-                traceback.print_tb(tb)
+                traceback.print_tb(tb, sys.stdout)
                 pattern = os.path.join(self.artifacts_dir, self.hashstring)
                 files_matching = glob.glob(pattern)
                 if len(files_matching) > 0:
@@ -305,26 +346,16 @@ class Artifact(object):
                         print f
                 raise e
 
-            h = hashlib.sha512()
-
             if self.data_dict and len(self.data_dict) > 0:
-                h.update(self.output_text().encode("utf-8"))
+                pass
 
             elif self.is_canonical_output_cached:
                 self.state = 'complete'
                 self.save()
 
-                f = open(self.filepath(), "rb")
-                while True:
-                    data = f.read(h.block_size)
-                    if not data:
-                        break
-                    h.update(data)
-
             else:
                 raise Exception("data neither in memory nor on disk")
 
-            self.output_hash = h.hexdigest()
             self.logstream = self.doc.logstream.getvalue()
             self.state = 'complete'
             self.source = 'run'
@@ -360,6 +391,7 @@ class Artifact(object):
         new_artifact.artifacts_dir = self.artifacts_dir
         new_artifact.inode = self.hashstring
         new_artifact.created_by = self.document_key
+        new_artifact.virtual = True
         # TODO filter class source?
 
         # TODO this is duplicated in setup_from_previous_artifact, should reorganize
@@ -507,8 +539,16 @@ class Artifact(object):
     def convert_if_not_unicode(self, s):
         if type(s) == unicode:
             return s
+        elif s == None:
+            return u""
         else:
-            return unicode(s, encoding="utf-8")
+            try:
+                ut = unicode(s, encoding="utf-8")
+                return ut
+            except Exception as e:
+                print "error occurred trying to convert text to unicode in", self.key
+                raise e
+
 
     def input_text(self):
         return u"".join([self.convert_if_not_unicode(v) for k, v in self.input_data_dict.items()])
@@ -583,6 +623,14 @@ class Artifact(object):
                 os.makedirs(os.path.dirname(workfile))
             shutil.copyfile(previous, workfile)
 
+    def alias(self):
+        """
+        Whether this artifact includes an alias.
+        """
+        aliases = [k for k in self.key.split("|") if k.startswith("-")]
+        if len(aliases) > 0:
+            return aliases[0]
+
     def canonical_dir(self, ignore_args = False):
         return os.path.dirname(self.name)
 
@@ -597,11 +645,19 @@ class Artifact(object):
             return os.path.join(parent_dir, self.args['canonical-name'])
         elif self.args.has_key('postfix') and not ignore_args:
             return "%s%s%s" % (fn, self.ext, self.args['postfix'])
+        elif self.alias():
+            return "%s%s%s" % (fn, self.alias(), self.ext)
         else:
             return "%s%s" % (fn, self.ext)
 
     def long_canonical_filename(self):
-        return "%s%s" % (self.key.replace("|", "-"), self.ext)
+        if not "|" in self.key:
+            return self.key.replace("|", "-")
+        else:
+            return "%s%s" % (self.key.replace("|", "-"), self.ext)
+
+    def websafe_key(self):
+        return self.long_canonical_filename().replace("/", "--")
 
     def filename(self):
         """
