@@ -22,6 +22,7 @@ class Controller(object):
         self.config = {} # config to be processed from .dexy files
         self.docs = []
         self.timing = []
+        self.virtual_docs = []
 
         self.batch_start_time = None
         self.batch_finish_time = None
@@ -29,7 +30,7 @@ class Controller(object):
 
         # Set up logging
         if args.has_key("logsdir") and args.has_key("logfile"):
-            self.log = get_log("dexy.controller", args['logsdir'], args['logfile'])
+            self.log = get_log("dexy.controller", args['logsdir'], args['logfile'], args['loglevel'])
         else:
             self.log = Constants.NULL_LOGGER
 
@@ -52,6 +53,7 @@ class Controller(object):
             else:
                 raise Exception("Artifact class name %s not found in %s" % (args['artifactclass'], ",".join(self.artifact_classes.keys())))
 
+
     def run(self):
         """
         This does all the work.
@@ -76,7 +78,11 @@ class Controller(object):
         self.timing.append(("process-config", time.time() - start))
         start = time.time()
 
+        # set the list of documents which are virtual
+        self.virtual_docs = [d for d in self.docs if d.virtual]
+
         if not self.args['dryrun']:
+            [doc.setup() for doc in self.docs]
             self.docs = [doc.run() for doc in self.docs]
         self.timing.append(("run-docs", time.time() - start))
 
@@ -119,6 +125,7 @@ class Controller(object):
         """
         global_args = {}
         config_dict = {}
+        variables = {}
         config_file = self.args['config']
 
         path_elements = path.split(os.sep)
@@ -154,7 +161,11 @@ class Controller(object):
                 if json_dict.has_key("$globals"):
                     global_args.update(json_dict["$globals"])
 
+                if json_dict.has_key("$variables"):
+                    variables.update(json_dict["$variables"])
+
         config_dict['$globals'] = global_args
+        config_dict['$variables'] = variables
         return config_dict
 
     def load_config(self):
@@ -226,6 +237,7 @@ class Controller(object):
 
         # Define the parse_doc nested function which we will call recursively.
         def parse_doc(path, input_directive, args = {}):
+            self.log.debug("Parsing doc path: '%s' input_dir: '%s' args : '%s'" % (path, input_directive, args))
             # If a specification is nested in a dependency, then input_directive
             # may be a dict. If so, split it into parts before continuing.
             try:
@@ -349,6 +361,8 @@ re.compile: %s""" % (args['except'], e))
                     if args.has_key('filters'):
                         doc.filters += args['filters']
 
+                    if args.has_key('loglevel'):
+                        doc.loglevelname = args['loglevel']
                     doc.setup_log() # After name has been set
                     doc.virtual = virtual
 
@@ -365,7 +379,10 @@ re.compile: %s""" % (args['except'], e))
                         doc.priority = args['priority']
                         del args['priority']
 
+                    self.log.debug("Doc args: '%s'" % doc.args)
+                    self.log.debug("updating with: '%s'" % args)
                     doc.args.update(args)
+                    self.log.debug("Doc args after update: '%s'" % doc.args)
 
                     if args.has_key('allinputs'):
                         doc.use_all_inputs = args['allinputs']
@@ -403,12 +420,18 @@ re.compile: %s""" % (args['except'], e))
             else:
                 global_args = {}
 
+            if config.has_key("$variables"):
+                global_variables = config["$variables"]
+            else:
+                global_variables = {}
+
             if self.args.has_key('globals'):
                 global_args.update(self.args['globals'])
 
             for k, v in config.iteritems():
                 local_args = global_args.copy()
                 local_args.update(v)
+                local_args['$variables'] = global_variables
                 for kg in global_args.keys():
                     if local_args.has_key(kg):
                         if isinstance(local_args[kg], dict):
