@@ -1,6 +1,8 @@
 import dexy.utils
+import shutil
 import platform
 import subprocess
+import os
 
 class DexyFilterException(Exception):
     pass
@@ -13,9 +15,8 @@ class DexyFilter(object):
     ALIASES in each handler, use java-style namespacing, e.g. com.abc.alias
     """
     ALIASES = ['dexy']
-    BINARY = False
+    BINARY = None
     FINAL = None
-    IGNORE_ERRORS = False # Set to True for filters where proc.returncode doesn't behave well
     INPUT_EXTENSIONS = [".*"]
     OUTPUT_EXTENSIONS = [".*"]
     TAGS = [] # Descriptive keywords about the filter.
@@ -165,9 +166,17 @@ class DexyFilter(object):
             method_used = "process_text_to_dict"
 
         if not method_used:
-            # This code implements the neutral 'dexy' handler.
-            self.artifact.data_dict = self.artifact.input_data_dict
+            if self.artifact.input_data_dict:
+                # This code implements the neutral 'dexy' handler.
+                if self.artifact.binary_output:
+                    shutil.copyfile(self.artifact.previous_artifact_filepath, self.artifact.filepath())
+                else:
+                    self.artifact.data_dict = self.artifact.input_data_dict
+            else:
+                if os.path.exists(self.artifact.previous_artifact_filepath) and not os.path.exists(self.artifact.filepath()):
+                    shutil.copyfile(self.artifact.previous_artifact_filepath, self.artifact.filepath())
+
             method_used = "process"
 
         return method_used
- 
+
