@@ -57,7 +57,15 @@ class Document(object):
 
     def setup_log(self):
         self.log = logging.getLogger(self.key())
-        self.log.setLevel(logging.DEBUG) # TODO Allow this to be changed.
+
+        if hasattr(self, 'loglevelname'):
+            loglevelname = self.loglevelname
+        elif self.controller.args.has_key('loglevel'):
+            loglevelname = self.controller.args['loglevel']
+        else:
+            loglevelname = Constants.DEFAULT_LOGLEVEL
+
+        self.log.setLevel(Constants.LOGLEVELS[loglevelname])
         self.log.propagate = 0 # Stops logs being written to STDOUT if another library redirects root logger.
         handler = logging.StreamHandler(self.logstream)
         formatter = logging.Formatter(Constants.DEFAULT_LOGFORMAT)
@@ -76,10 +84,10 @@ class Document(object):
         self.filters += filters
 
     def get_filter_for_alias(self, alias):
-        """
-        Given a filter alias, return the corresponding filter class.
-        """
-        return dexy.introspect.get_filter_for_alias(alias, self.__class__.filter_list)
+        if alias == '':
+            raise Exception("You have a trailing | or 2 | symbols together in your specification for %s" % self.key())
+        else:
+            return dexy.introspect.get_filter_for_alias(alias, self.__class__.filter_list)
 
     def final_artifact(self):
         return self.last_artifact
@@ -318,28 +326,34 @@ class Document(object):
         artifact.save()
         if artifact.binary_output:
             if artifact.doc.virtual:
-                # make a fake file
-                with open(artifact.filepath(), "wb") as f:
-                    f.write("virtual file")
+                if not os.path.exists(artifact.filepath()):
+                    # make a fake file
+                    with open(artifact.filepath(), "wb") as f:
+                        f.write("virtual file")
             else:
                 shutil.copyfile(artifact.name, artifact.filepath())
         return artifact
 
-    def run(self):
-        self.step = 0
+    def setup(self):
+        self.virtual_docs = self.controller.virtual_docs
+
         start = time.time()
-        time_start = start
-
-
+        self.step = 0
         artifact = self.create_initial_artifact()
         self.artifacts.append(artifact)
         self.last_artifact = artifact
         artifact_key = artifact.key
         self.timing.append(("create-initial-artifact", time.time() - start))
-        start = time.time()
         self.log.info("(step %s) [run] %s -> %s" % \
                  (self.step, artifact_key, artifact.filename()))
 
+    def run(self):
+        start = time.time()
+        time_start = start
+
+        artifact = self.artifacts[0]
+        artifact_key = artifact.key
+
         for f in self.filters:
             previous_artifact = artifact
             artifact_key += "|%s" % f
