@@ -2,24 +2,49 @@ from dexy.dexy_filter import DexyFilter
 import os
 import subprocess
 
+class DexyScriptErrorException(Exception):
+    pass
+
+class DexyNonzeroExitException(DexyScriptErrorException):
+    def __init__(self, command, exitcode, stderr):
+        msg = "'%s' returned nonzero exit code %s." % (command, exitcode)
+        msg += " Set --loglevel DEBUG to troubleshoot or run with --ignore or set ignore-errors : true for this document to skip error-checking."
+        if stderr and not stderr=='':
+            msg += " Error info: %s" % stderr
+        Exception.__init__(self,  msg)
+
+class DexyEOFException(DexyScriptErrorException):
+    pass
+
 class ProcessFilter(DexyFilter):
     """
     Base class for all classes that do external processing, via subprocess or pexpect.
     """
     ALIASES = ['processfilter']
+    CHECK_RETURN_CODE = True
 
-    def handle_subprocess_proc_return(self, returncode, stderr):
-        if returncode is None:
+    def ignore_errors(self):
+        """
+        By default, dexy filters should raise an exception when a script they
+        run has a nonzero exit code or an EOF. Sometimes you want to document
+        code that has an error, so you can disable this error checking.
+
+        For software which does not consistently return nonzero error codes,
+        you can set CHECK_RETURN_CODE to False.
+        """
+        artifact_ignore = self.artifact.args.has_key('ignore-errors') and self.artifact.args['ignore-errors']
+        controller_ignore = self.artifact.controller_args['ignore']
+        return artifact_ignore or controller_ignore
+
+    def handle_subprocess_proc_return(self, command, exitcode, stderr):
+        if exitcode is None:
             raise Exception("no return code, proc not finished!")
-        elif returncode != 0:
-            artifact_ignore = self.artifact.args.has_key('ignore') and self.artifact.args['ignore']
-            controller_ignore = self.artifact.controller_args['ignore']
-            if self.IGNORE_ERRORS or artifact_ignore or controller_ignore:
-                self.artifact.log.warn("Nonzero exit status %s" % returncode)
+        elif exitcode != 0 and self.CHECK_RETURN_CODE:
+            if self.ignore_errors():
+                self.artifact.log.warn("Nonzero exit status %s" % exitcode)
                 self.artifact.log.warn("output from process: %s" % stderr)
             else:
-                print stderr
-                raise Exception("""proc returned nonzero status code! if you don't want dexy to raise errors on failed scripts then pass the -ignore option""")
+                raise DexyNonzeroExitException(command, exitcode, stderr)
 
     def setup_env(self):
         if self.artifact.args.has_key('env'):
@@ -87,6 +112,15 @@ class ProcessFilter(DexyFilter):
         }
         return "%(prog)s %(args)s %(script_file)s %(scriptargs)s" % args
 
+    def clean_nonprinting(self, text):
+        proc = subprocess.Popen('col -b | strings', shell=True,
+                                stdin=subprocess.PIPE,
+                                stdout=subprocess.PIPE,
+                                stderr=subprocess.STDOUT)
+
+        stdout, stderr = proc.communicate(text)
+        return stdout
+
 class SubprocessFilter(ProcessFilter):
     ALIASES = ['subprocessfilter']
     BINARY = True
@@ -95,7 +129,7 @@ class SubprocessFilter(ProcessFilter):
     def process(self):
         command = self.command_string()
         proc, stdout = self.run_command(command, self.setup_env())
-        self.handle_subprocess_proc_return(proc.returncode, stdout)
+        self.handle_subprocess_proc_return(command, proc.returncode, stdout)
         self.artifact.stdout = stdout
 
     def run_command(self, command, env, input_text = None):
@@ -108,7 +142,11 @@ class SubprocessFilter(ProcessFilter):
                                 stderr=subprocess.STDOUT,
                                 env=env)
 
+        if input_text:
+            self.log.debug("about to send input '%s'" % input_text)
+
         stdout, stderr = proc.communicate(input_text)
+        self.log.debug(stdout)
         return (proc, stdout)
 
 class SubprocessStdoutFilter(SubprocessFilter):
@@ -118,9 +156,9 @@ class SubprocessStdoutFilter(SubprocessFilter):
 
     def process(self):
         command = self.command_string_stdout()
-        self.log.debug("About to run '%s'" % command)
         proc, stdout = self.run_command(command, self.setup_env())
-        self.handle_subprocess_proc_return(proc.returncode, stdout)
+        if self.CHECK_RETURN_CODE:
+            self.handle_subprocess_proc_return(command, proc.returncode, stdout)
         self.artifact.set_data(stdout)
 
 class SubprocessStdoutInputFilter(SubprocessFilter):
@@ -131,16 +169,20 @@ class SubprocessStdoutInputFilter(SubprocessFilter):
     def process(self):
         command = self.command_string_stdout()
 
-        if len(self.artifact.inputs()) == 1:
+        if len(self.artifact.inputs()) == 0:
+            raise Exception("Trying to call an input filter without supplying any inputs!")
+
+        elif len(self.artifact.inputs()) == 1:
             artifact = self.artifact.inputs().values()[0]
             for section_name, section_text in artifact.data_dict.iteritems():
                 proc, stdout = self.run_command(command, self.setup_env(), section_text)
-                self.handle_subprocess_proc_return(proc.returncode, stdout)
+                self.handle_subprocess_proc_return(command, proc.returncode, stdout)
                 self.artifact.data_dict[section_name] = stdout
+
         else:
             for artifact in self.artifact.inputs().values():
                 proc, stdout = self.run_command(command, self.setup_env(), artifact.output_text())
-                self.handle_subprocess_proc_return(proc.returncode, stdout)
+                self.handle_subprocess_proc_return(command, proc.returncode, stdout)
                 rel_key = self.artifact.relative_key_for_input(artifact)
                 self.artifact.data_dict[rel_key] = stdout
 
@@ -159,7 +201,7 @@ class SubprocessStdoutInputFileFilter(SubprocessFilter):
         for artifact in self.artifact.inputs().values():
             command = self.command_string_stdout_input(artifact)
             proc, stdout = self.run_command(command, self.setup_env())
-            self.handle_subprocess_proc_return(proc.returncode, stdout)
+            self.handle_subprocess_proc_return(command, proc.returncode, stdout)
             rel_key = self.artifact.relative_key_for_input(artifact)
             self.artifact.data_dict[rel_key] = stdout
 
@@ -186,12 +228,16 @@ class SubprocessCompileFilter(SubprocessFilter):
         # Compile the code
         command = self.compile_command_string()
         proc, stdout = self.run_command(command, self.setup_env())
-        self.handle_subprocess_proc_return(proc.returncode, stdout)
+        # This tests exitcode from the *compiler*
+        self.handle_subprocess_proc_return(command, proc.returncode, stdout)
 
+        # Run the compiled code
         command = self.run_command_string()
         proc, stdout = self.run_command(command, self.setup_env())
+
+        # This tests exitcode from the compiled script.
         if self.CHECK_RETURN_CODE:
-            self.handle_subprocess_proc_return(proc.returncode, stdout)
+            self.handle_subprocess_proc_return(command, proc.returncode, stdout)
 
         self.artifact.set_data(stdout)
 
@@ -203,7 +249,7 @@ class SubprocessCompileInputFilter(SubprocessCompileFilter):
         # Compile the code
         command = self.compile_command_string()
         proc, stdout = self.run_command(command, self.setup_env())
-        self.handle_subprocess_proc_return(proc.returncode, stdout)
+        self.handle_subprocess_proc_return(command, proc.returncode, stdout)
 
         command = self.run_command_string()
 
@@ -212,12 +258,12 @@ class SubprocessCompileInputFilter(SubprocessCompileFilter):
             for section_name, section_text in artifact.data_dict.iteritems():
                 proc, stdout = self.run_command(command, self.setup_env(), section_text)
                 if self.CHECK_RETURN_CODE:
-                    self.handle_subprocess_proc_return(proc.returncode, stdout)
+                    self.handle_subprocess_proc_return(command, proc.returncode, stdout)
                 self.artifact.data_dict[section_name] = stdout
         else:
             for artifact in self.artifact.inputs().values():
                 proc, stdout = self.run_command(command, self.setup_env(), artifact.output_text())
                 if self.CHECK_RETURN_CODE:
-                    self.handle_subprocess_proc_return(proc.returncode, stdout)
+                    self.handle_subprocess_proc_return(command, proc.returncode, stdout)
                 rel_key = self.artifact.relative_key_for_input(artifact)
                 self.artifact.data_dict[rel_key] = stdout
