@@ -1,6 +1,41 @@
 from dexy.filters.process_filters import SubprocessStdoutFilter
 from dexy.filters.process_filters import SubprocessStdoutInputFilter
 from dexy.filters.process_filters import SubprocessStdoutInputFileFilter
+import json
+
+class CleanSubprocessStdoutFilter(SubprocessStdoutFilter):
+    """
+    Clean non-printing characters from text using the 'strings' tool.
+    """
+    ALIASES = ['clean', 'strings']
+    EXECUTABLE = 'strings'
+
+class ManPageSubprocessStdoutFilter(SubprocessStdoutFilter):
+    """
+    Read command names from a file and fetch man pages for each.
+
+    Returns a JSON dict whose keys are the program names and values are man
+    pages.
+    """
+    ALIASES = ['man']
+    EXECUTABLE = 'man'
+    VERSION_COMMAND = 'man --version'
+    INPUT_EXTENSIONS = [".txt"]
+    OUTPUT_EXTENSIONS = [".json"]
+
+    def command_string(self, prog_name):
+        # Use bash rather than the default of sh (dash) so we can set pipefail.
+        return "bash -c \"set -e; set -o pipefail; man %s | col -b | strings\"" % (prog_name)
+
+    def process(self):
+        man_info = {}
+        for prog_name in self.artifact.input_text().split():
+            command = self.command_string(prog_name)
+            proc, stdout = self.run_command(command, self.setup_env())
+            self.handle_subprocess_proc_return(command, proc.returncode, stdout)
+            man_info[prog_name] = stdout
+
+        self.artifact.set_data(json.dumps(man_info))
 
 class RhinoSubprocessStdoutFilter(SubprocessStdoutFilter):
     EXECUTABLE = "rhino -f"
@@ -39,14 +74,14 @@ class PythonSubprocessStdoutInputFilter(SubprocessStdoutInputFilter):
 
 class BashSubprocessStdoutFilter(SubprocessStdoutFilter):
     ALIASES = ['sh', 'bash']
-    EXECUTABLE = 'bash'
+    EXECUTABLE = 'bash -e'
     INPUT_EXTENSIONS = [".sh", ".bash", ".txt", ""]
     OUTPUT_EXTENSIONS = [".txt"]
     VERSION_COMMAND = 'bash --version'
 
 class BashSubprocessStdoutInputFilter(SubprocessStdoutInputFilter):
     ALIASES = ['shinput']
-    EXECUTABLE = 'bash'
+    EXECUTABLE = 'bash -e'
     INPUT_EXTENSIONS = [".sh", ".bash", ".txt", ""]
     OUTPUT_EXTENSIONS = [".txt"]
     VERSION_COMMAND = 'bash --version'
@@ -75,6 +110,7 @@ class RegetronSubprocessStdoutInputFileFilter(SubprocessStdoutInputFileFilter):
 
 class IrbSubprocessStdoutFilter(SubprocessStdoutFilter):
     ALIASES = ['irbout']
+    CHECK_RETURN_CODE = False
     EXECUTABLE = 'irb --simple-prompt --noreadline'
     INPUT_EXTENSIONS = [".txt", ".rb"]
     OUTPUT_EXTENSIONS = [".rbcon"]
@@ -86,6 +122,7 @@ class IrbSubprocessStdoutInputFilter(SubprocessStdoutInputFilter):
     INPUT_EXTENSIONS = [".txt", ".rb"]
     OUTPUT_EXTENSIONS = [".rbcon"]
     VERSION_COMMAND = 'irb --version'
+    CHECK_RETURN_CODE = False
 
 class RubySubprocessStdoutFilter(SubprocessStdoutFilter):
     EXECUTABLE = 'ruby'
@@ -117,6 +154,7 @@ class PhpFilter(SubprocessStdoutFilter):
     Runs php script and returns STDOUT.
     """
     EXECUTABLE = 'php'
+    CHECK_RETURN_CODE = False # TODO see if it's possible to check return code for PHP
     VERSION_COMMAND = 'php --version'
     INPUT_EXTENSIONS = [".php"]
     OUTPUT_EXTENSIONS = [".html", ".txt"]
