@@ -5,6 +5,39 @@ import re
 import shutil
 import subprocess
 
+class PandocFilter(SubprocessFilter):
+    EXECUTABLE = "pandoc"
+    VERSION_COMMAND = "pandoc --version"
+    ALIASES = ['pandoc']
+    OUTPUT_EXTENSIONS = ['.html', '.txt', '.tex', '.pdf', '.rtf', '.json', '.docx']
+    BINARY = None # determine binary from file extension
+
+    def command_string(self):
+        args = {
+            'prog' : self.executable(),
+            'args' : self.command_line_args() or "",
+            'script_file' : self.artifact.previous_artifact_filename,
+            'output_file' : self.artifact.filename()
+        }
+        return "%(prog)s %(args)s %(script_file)s -o %(output_file)s" % args
+
+class EspeakFilter(SubprocessFilter):
+    EXECUTABLE = "espeak"
+    INPUT_EXTENSIONS = [".txt"]
+    OUTPUT_EXTENSIONS = [".wav"]
+    ALIASES = ['espeak']
+    BINARY = True
+
+    def command_string(self):
+        args = {
+            'prog' : self.executable(),
+            'args' : self.command_line_args() or "",
+            'scriptargs' : self.command_line_scriptargs() or "",
+            'script_file' : self.artifact.previous_artifact_filename,
+            'output_file' : self.artifact.filename()
+        }
+        return "%(prog)s %(args)s -w %(output_file)s %(script_file)s" % args
+
 class AsciidocFilter(SubprocessFilter):
     VERSION_COMMAND = "asciidoc --version"
     EXECUTABLE = "asciidoc"
@@ -188,12 +221,14 @@ class EmbedFonts(SubprocessFilter):
     def process(self):
         env = self.setup_env()
 
-        proc, stdout = self.run_command(self.preprocess_command_string(), env)
-        self.handle_subprocess_proc_return(proc.returncode, stdout)
+        command = self.preprocess_command_string()
+        proc, stdout = self.run_command(command, env)
+        self.handle_subprocess_proc_return(command, proc.returncode, stdout)
         self.artifact.stdout = stdout
 
-        proc, stdout = self.run_command(self.pdffonts_command_string(), env)
-        self.handle_subprocess_proc_return(proc.returncode, stdout)
+        command = self.pdffonts_command_string()
+        proc, stdout = self.run_command(command, env)
+        self.handle_subprocess_proc_return(command, proc.returncode, stdout)
         self.artifact.stdout += stdout
 
 class ROutputBatchFilter(SubprocessFilter):
@@ -335,7 +370,7 @@ class Pdf2ImgSubprocessFilter(SubprocessFilter):
         command = self.command_string()
         proc, stdout = self.run_command(command, self.setup_env())
         self.artifact.stdout = stdout
-        self.handle_subprocess_proc_return(proc.returncode, stdout)
+        self.handle_subprocess_proc_return(command, proc.returncode, stdout)
 
         if self.artifact.args.has_key('page'):
             page = self.artifact.args['page']
