@@ -48,9 +48,15 @@ class PygmentsStylesheetTemplatePlugin(TemplatePlugin):
 
     def run(self):
         pygments_stylesheets = {}
+        if self.filter_instance.artifact.args.has_key('pygments'):
+            formatter_args = self.filter_instance.artifact.args['pygments']
+        else:
+            formatter_args = {}
+
         for style_name in get_all_styles():
             for formatter_class in [pygments.formatters.LatexFormatter, pygments.formatters.HtmlFormatter]:
-                pygments_formatter = formatter_class(style=style_name)
+                formatter_args['style'] = style_name
+                pygments_formatter = formatter_class(**formatter_args)
                 style_info = pygments_formatter.get_style_defs()
 
                 for fn in pygments_formatter.filenames:
@@ -72,12 +78,18 @@ class SubdirectoriesTemplatePlugin(TemplatePlugin):
 
 class VariablesTemplatePlugin(TemplatePlugin):
     def run(self):
+        variables = {}
         if self.filter_instance.artifact.args.has_key('variables'):
-            return self.filter_instance.artifact.args['variables']
-        else:
-            return {}
+            variables.update(self.filter_instance.artifact.args['variables'])
+        if self.filter_instance.artifact.args.has_key('$variables'):
+            variables.update(self.filter_instance.artifact.args['$variables'])
+        return variables
 
 class GlobalsTemplatePlugin(TemplatePlugin):
+    """
+    Makes available the global variables specified on the dexy command line
+    using the --globals option
+    """
     def run(self):
         if self.filter_instance.artifact.controller_args.has_key('globals'):
             return self.filter_instance.artifact.controller_args['globals']
@@ -102,12 +114,16 @@ class InputsTemplatePlugin(TemplatePlugin):
                     od[k] = v
             return od
 
-        return sort_dict(unsorted_json)
+        if type(unsorted_json) == dict:
+            return sort_dict(unsorted_json)
+        else:
+            return unsorted_json
 
     def run(self):
         d_hash = {}
         a_hash = {}
 
+
         name = self.filter_instance.artifact.name
         inputs = self.filter_instance.artifact.inputs()
 
@@ -127,7 +143,7 @@ class InputsTemplatePlugin(TemplatePlugin):
 
                 a_hash[k] = a
 
-                if data.keys() == ['1']:
+                if hasattr(data, 'keys') and data.keys() == ['1']:
                     d_hash[k] = data['1']
                 else:
                     d_hash[k] = data
